<?php
namespace Teknisa\Libs\Util;


use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Security\OAuth\OAuthImpl;
use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Service\Api;

class RequestListenerOAuth extends Listener {

    const OAUTH_HEADER = "OAuth-Token";
    const OAUTH_SECRET_ID = "OAuth-SecretId";
    /**
     * @var \Zeedhi\Framework\HTTP\Kernel Kernel
     */
    private $httpKernel;
    private $oauth;
    private $sessionManager;
    private $apiService;
    private $urls;

    public function __construct(Kernel $kernel, OAuthImpl $oauth, Session $sessionManager, Api $apiService){
        $this->httpKernel = $kernel;
        $this->oauth = $oauth;
        $this->sessionManager = $sessionManager;
        $this->apiService = $apiService;
        $this->urls = $this->readPublicRoutes();
    }

    private function readPublicRoutes(){
        $urls = file_get_contents(__DIR__ . "/../../../../config/publicRoutes.json");
        $publicRoutes = json_decode($urls, true);
        $productUrls = @file_get_contents(Utilities::getProductBasePath() . "/backend/config/publicRoutes.json");
        if(!empty($productUrls)) {
            $publicRoutes = array_merge($publicRoutes, json_decode($productUrls, true));
        }
        return $publicRoutes;
    }

    public function preDispatch(Request $request)
    {        
        try {
            $this->apiService->setLastRequestTime($request->getRoutePath());
            $token = $this->httpKernel->getHttpRequest()->getHeaders()->get(self::OAUTH_HEADER);
            if (!is_null($token)) {
                $secretId = $this->httpKernel->getHttpRequest()->getHeaders()->get(self::OAUTH_SECRET_ID);
                $service = $this->oauth->checkAccess($token, $secretId);
                if(!empty($service)) {
                    $this->apiService->setSession($service->getNrOrg(), $service->getCdOperator());
                }
            }

            if (!in_array($request->getRoutePath(), $this->urls) && !$this->sessionManager->has("NRORG")) {
                throw new \Exception("OAuth Access denied.");
            }
        } catch (\Exception $e){
            throw new \Exception($e->getMessage());
        }
    }
}